/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: in 20 Minutes (Coffee Break Series)
	 Beginners Taste

     Minimal working examples (MWE) to build upon and 
	 modify to gain an understanding of the API

	 Extensions to the Vulkan Graphics Coffee Break Series
	   
/*
/****************************************************************/

/*
	Simple geometry shader
*/

#version 420


//layout (points) in;
//layout (triangle_strip, max_vertices=4) out;

layout(triangles) in;
layout(line_strip, max_vertices=32) out;


in VertexData 
{
	vec4  Position;
	vec2  UV;
	vec3  Color;
	vec3  Normal;
} VertexIn[];

out VertexData 
{
	vec4  Position;
	vec2  UV;
	vec3  Color;
	vec3  Normal;
} VertexOut;


layout (binding = 0) uniform UBO 
{
	mat4  model;
	mat4  view;
	mat4  projection;
} ubo;

void main()
{

	for(int i=0; i<gl_in.length(); i++)
	{
		vec3 Pa = gl_in[(i+0)%3].gl_Position.xyz;
		vec3 Pb = gl_in[(i+1)%3].gl_Position.xyz;
    
		// Wireframe original triangle
		gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pa, 1.0);
		VertexOut.Position = gl_Position;
		VertexOut.UV      = vec2(0,0);
		VertexOut.Color    = vec3(1, 1, 0);
		EmitVertex();
    
		gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pb, 1.0);
		VertexOut.Position = gl_Position;
		VertexOut.UV       = vec2(0,0);
		VertexOut.Color    = vec3(1, 1, 0);
		EmitVertex();
		EndPrimitive();
	}



  
  //---Draw normal for each face (in middle of the triangle)
  //
  vec3 P0 = gl_in[0].gl_Position.xyz;
  vec3 P1 = gl_in[1].gl_Position.xyz;
  vec3 P2 = gl_in[2].gl_Position.xyz;
  
  /*
  // If we want to calculate the normal
  vec3 V0 = P0 - P1;
  vec3 V1 = P2 - P1;
  
  vec3 N = cross(V1, V0);
  N = normalize(N);
  */

  vec3 N = VertexIn[0].Normal.xyz;

  // Center of the triangle
  vec3 P = (P0+P1+P2) / 3.0;
  
  gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(P, 1.0);
  VertexOut.Position = gl_Position;
  VertexOut.UV       = vec2(0,0);
  VertexOut.Color    = vec3(1, 0, 0);
  EmitVertex();
  
  gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(P + N * 0.3, 1.0);
  VertexOut.Position = gl_Position;
  VertexOut.UV       = vec2(0,0);
  VertexOut.Color    = vec3(1, 0, 0);
  EmitVertex();
  EndPrimitive();
  

/*
	vec3 Pos = gl_in[0].gl_Position.xyz;

	mat4 mv = ubo.view * ubo.model;

	vec3 right = vec3( mv[0][0], mv[1][0], mv[2][0] );
	vec3 up    = vec3( mv[0][1], mv[1][1], mv[2][1] );
	

    Pos -= (right * 0.5);
    gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pos, 1.0);
	VertexOut.Position = gl_Position;
    VertexOut.UV = vec2(0.0, 0.0);
    EmitVertex();

    Pos += up;
    gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pos, 1.0);
	VertexOut.Position = gl_Position;
    VertexOut.UV = vec2(0.0, 1.0);
    EmitVertex();

    Pos -= up;
    Pos += right;
    gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pos, 1.0);
	VertexOut.Position = gl_Position;
    VertexOut.UV = vec2(1.0, 0.0);
    EmitVertex();

    Pos += up;
    gl_Position = (ubo.projection * ubo.view * ubo.model) * vec4(Pos, 1.0);
	VertexOut.Position = gl_Position;
    VertexOut.UV = vec2(1.0, 1.0);
    EmitVertex();

    EndPrimitive();
*/


	/*
	// Pass through geometry shader example
	layout(triangles) in;
	layout (triangle_strip, max_vertices=3) out;
	for(int i = 0; i < gl_in.length(); i++)
	{
		// copy attributes
		gl_Position = gl_in[i].gl_Position;
		VertexOut.Position  = VertexIn[i].Position;
		VertexOut.UV		= VertexIn[i].UV;
		VertexOut.Color		= VertexIn[i].Color;
		VertexOut.Normal	= VertexIn[i].Normal;

		// done with the vertex
		EmitVertex();
	}
	EndPrimitive();
	*/

	/*
	// Copies
	for (int x=0; x< 2; ++x)
	{
		for (int z=0; z<2; ++z)
		{
			// layout in is `triangles' - this means gl_in.length() should be 3
			// (i.e., vertices of a triangle)

			for(int i = 0; i < gl_in.length(); i++)
			{
				// copy attributes and displace copy
				//gl_Position = gl_in[i].gl_Position;

				gl_Position  = ubo.projection * ubo.view * ubo.model * gl_in[i].gl_Position;

				VertexOut.Position  = VertexIn[i].Position;
				VertexOut.UV		= VertexIn[i].UV;
				VertexOut.Color		= VertexIn[i].Color;
				VertexOut.Normal	= VertexIn[i].Normal;

				// done with the vertex
				EmitVertex();
			}
			EndPrimitive();
		}
	}
	*/
}